/* 
 * Copyright (C) 2004 Romain Dolbeau <romain@dolbeau.org>
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of this archive for
 *  more details.
 */

package org.dolbeau.xsCOEj;

import java.util.*;
import javax.swing.*;
import java.awt.event.*;
import org.jfree.chart.*;
import org.jfree.data.*;
import org.jfree.chart.plot.*;
import org.jfree.chart.labels.*;
import org.jfree.chart.renderer.*;
import java.awt.Color;
import java.awt.Font;

public class OptionDisplayer extends JPanel {
    private final Option myOption;
    private ChartPanel pieChartPanel = null;
    private ChartPanel lineChartPanel = null;
    private DefaultCategoryDataset lineCharDataset = new DefaultCategoryDataset();
    
    public OptionDisplayer(Option m) {
        myOption = m;
        pieChartPanel = new ChartPanel(createOptionPieChart(new DefaultPieDataset(), myOption.getDescription()));
        pieChartPanel.setPreferredSize(new java.awt.Dimension(360, 240));
        add(pieChartPanel);
        lineChartPanel = new ChartPanel(createLineChart(lineCharDataset, myOption.getDescription()));
        lineChartPanel.setPreferredSize(new java.awt.Dimension(400, 240));
        add(lineChartPanel);

        /*
        List l = myOption.getAllInstances();
        Integer iStep = new Integer(0);
        Double number = new Double(100./(double)l.size());
        Iterator it = l.iterator();
        while (it.hasNext()) {
            OptionInstance loi = (OptionInstance)it.next();
            lineCharDataset.addValue(number, loi.toString(), iStep);
        }
        */
    }

    private JFreeChart createOptionPieChart(PieDataset dataset, String d) {
        final JFreeChart pieChart = ChartFactory.createPieChart(d,
                                                                dataset,
                                                                false,
                                                                true,
                                                                false);
        final PiePlot piePlot = (PiePlot)pieChart.getPlot();
        piePlot.setNoDataMessage("No data available");
        piePlot.setCircular(false);
        piePlot.setLabelLinkPaint(Color.red);
        piePlot.setLabelGap(0.02);
        piePlot.setLabelGenerator(new StandardPieItemLabelGenerator("\"{0}\"={1} ({2})",
                                                                    java.text.NumberFormat.getNumberInstance(), 
                                                                    java.text.NumberFormat.getPercentInstance()));
        piePlot.setToolTipGenerator(new StandardPieItemLabelGenerator("\"{0}\"={1} ({2})",
                                                                    java.text.NumberFormat.getNumberInstance(), 
                                                                    java.text.NumberFormat.getPercentInstance()));
        return pieChart;
    }

    private JFreeChart createLineChart(CategoryDataset dataset, String d) {
        final JFreeChart lineChart = ChartFactory.createLineChart(d,
                                                              "Step",
                                                              "%",
                                                              dataset,
                                                              org.jfree.chart.plot.PlotOrientation.VERTICAL,
                                                              false,
                                                              false,
                                                              false);
        final CategoryPlot linePlot = (CategoryPlot)lineChart.getPlot();
        linePlot.setNoDataMessage("No data available");
        final org.jfree.chart.axis.NumberAxis rangeAxis = (org.jfree.chart.axis.NumberAxis)linePlot.getRangeAxis();
        rangeAxis.setStandardTickUnits(org.jfree.chart.axis.NumberAxis.createIntegerTickUnits());
        rangeAxis.setAutoRangeIncludesZero(true);
        return lineChart;
    }

    public void buildCharts(Map statMap, int step, int nst) {
        DefaultPieDataset dataset = new DefaultPieDataset();

        final JFreeChart pieChart = pieChartPanel.getChart();
        final PiePlot piePlot = (PiePlot)pieChart.getPlot();

        List l = myOption.getAllInstances();
        Integer iStep = new Integer(step);
        Iterator it = l.iterator();
        int j = 0;
        while (it.hasNext()) {
            OptionInstance loi = (OptionInstance)it.next();
            Integer i = (Integer)statMap.get(loi);
            if (i == null)
                i = new Integer(0);
            dataset.setValue(loi, i);

            lineCharDataset.addValue(new Double(100. * i.doubleValue()/(double)nst), loi.toString(), iStep);

            j++;
        }
        piePlot.setDataset(dataset);
    }
}
