/* 
 * Copyright (C) 2004 Romain Dolbeau <romain@dolbeau.org>
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of this archive for
 *  more details.
 */

package org.dolbeau.xsCOEj;

/**
 * The abstract Class representing an instance of an option of the command
 * @author Romain Dolbeau <romain@dolbeau.org>
 * @version  $Id: OptionInstance.java,v 1.3 2005/02/09 14:42:39 dolbeau Exp $
 * @see OptionInstance
 * @see OptionInstanceList
 */
public abstract class OptionInstance implements Comparable, java.io.Serializable {
    private static int next = 0;
    private final Option parent;

    /** Return the representation of this OptionInstance, as accepted by the command.
     * @return a String representing this OptionInstance, as accepted by the command.
     */
    public abstract String getOptionString();

    public OptionInstance(Option p) {
        parent = p;
    }

    public String toString() {
        return getOptionString();
    }
   
    public boolean equals(Object o) {
        if (!(o instanceof OptionInstance))
            return false;
        OptionInstance o2 = (OptionInstance)o;
        boolean e = getParentOption().equals(o2.getParentOption());
        if (!e)
            return e;
        if (getOptionString() == null) {
            if (o2.getOptionString() == null)
                return true;
            else
                return false;
        } else {
            if (o2.getOptionString() == null)
                return false;
            else
                return getOptionString().equals(o2.getOptionString());
        }
    }
 
    public int compareTo(Object o) {
        OptionInstance o2 = (OptionInstance)o;
        int n = getParentOption().compareTo(o2.getParentOption());
        if (n != 0)
            return n;
        if (getOptionString() == null) {
            if (o2.getOptionString() == null)
                return 0;
            else
                return -1;
        } else {
            if (o2.getOptionString() == null)
                return 1;
            else
                return getOptionString().compareTo(o2.getOptionString());
        }
    }
    
    public boolean isCompatible(OptionInstance oi) {
        return true;
    }

    public Option getParentOption() {
        return parent;
    }
}
