/* 
 * Copyright (C) 2004 Romain Dolbeau <romain@dolbeau.org>
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of this archive for
 *  more details.
 */

package org.dolbeau.xsCOEj;

import java.util.*;

public class OptionInstanceListBreeder1 {
    static class ComparatorByResults implements Comparator {
        private ResultsPool lrp;

        ComparatorByResults(ResultsPool l) {
            lrp = l;
        }

        public int compare(Object o1, Object o2) {
            OptionInstanceList l1 = (OptionInstanceList)o1;
            OptionInstanceList l2 = (OptionInstanceList)o2;
            Results r1 = lrp.getResult(l1);
            Results r2 = lrp.getResult(l2);
            return r2.compareTo(r1);
        }
    }

    private static OptionInstanceList breed(OptionInstanceList l1, OptionInstanceList l2, int oddsR, Set allOptions, Random random) {
        OptionInstanceList temp = new OptionInstanceList();
        Iterator it = allOptions.iterator();
        while (it.hasNext()) {
            Option opt = (Option)it.next();
            int i = 0;
            boolean done = false;
            do {
              OptionInstance oi = opt.breed(l1.getOptionInstance(opt), l2.getOptionInstance(opt), oddsR, random);
              if ((oi != null) && temp.isCompatible(oi)) {
                temp.add(oi);
                done = true;
              }
            } while (!done && i < ( OptionInstanceList.MAX_ATTEMPTS));
            if (!done)
              return null;
        }
        Collections.sort(temp);
        return temp;
    }

    public static Set breedThemAll(Set allButNKeys, int max, ResultsPool lrp, int oddsR, Set allOptions, Random random) {
        Set children = new TreeSet();
        int failed = 0;
        List allButN2 = new ArrayList(allButNKeys);
        // put the best results first
        Collections.sort(allButN2, new ComparatorByResults(lrp));

        for (int i = 0; (i < max) && (allButN2.size() > 0) ; i++) {
            int j = 0;
            OptionInstanceList c = null;
            do {
                OptionInstanceList l1 = null;
                OptionInstanceList l2 = null;
                int n1 = -1;
                int n2 = -1;
                // bias the first parent towards the head of the list (highest PSNR)
                int k = (2 * j ) + 3;
                // if we already have failed many attemps, remove the bias
                if (j > (OptionInstanceList.MAX_ATTEMPTS / 2))
                    k = allButN2.size();
                // we musn't go too far...
                if (k > allButN2.size())
                    k = allButN2.size();
                n1 = random.nextInt(k);
                n2 = random.nextInt(allButN2.size());
                l1 = (OptionInstanceList)allButN2.get(n1);
                l2 = (OptionInstanceList)allButN2.get(n2);
                c = breed(l1, l2, oddsR, allOptions, random);
                j++;
            } while (((c == null) || children.contains(c) || lrp.hasResult(c)) && (j <  OptionInstanceList.MAX_ATTEMPTS));
            failed += (j - 1);
            if (!((c == null) || children.contains(c) || lrp.hasResult(c))) {
                children.add(c);
            } else {
                failed++;
            }
        }
        System.err.println("Breeding: " + failed + " attempts failed.");
        return children;
    }

}