/* 
 * Copyright (C) 2004 Romain Dolbeau <romain@dolbeau.org>
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of this archive for
 *  more details.
 */

package org.dolbeau.xsCOEj;

import java.util.*;
import javax.swing.*;
import java.awt.event.*;

public class StatisticsDisplayPanel extends JPanel {
    private JFrame parent;
    private OptionDisplayersPanel lodp;
    private ResultsDisplay lBest;
    private ResultsDisplay lFastest;

    public StatisticsDisplayPanel(Set allOptions, JFrame p) {
        parent = p;

        lodp = new OptionDisplayersPanel(allOptions);

        lBest = new ResultsDisplay();

        lFastest = new ResultsDisplay();
        
        final Box box = Box.createVerticalBox();
        add(box);
        box.add(lodp);
        box.add(lBest);
        box.add(lFastest);

        parent.pack();
        parent.setVisible(true);
    }

    public void refresh(ResultsPool lrp,
                 OptionInstanceList best,
                 int step,
                 int bestStep,
                 OptionInstanceList fastest,
                 double p,
                 ResultsPool allResults) {
        boolean improved = (step != bestStep);
        lodp.updateAllDisplayers(lrp, step);

        lBest.setResults("Best so far", best, (Results)lrp.getResult(best), step, bestStep);
        lFastest.setResults("Fastest within " + p + " percent", fastest, (Results)allResults.getResult(fastest), -1, -1);
        parent.pack();
        parent.repaint();
    }

    public static StatisticsDisplayPanel createStatisticsDisplayPanel(Set allOptions) {
        final JFrame frame = new JFrame("xsCOEj Results");
        
        final StatisticsDisplayPanel cd = new StatisticsDisplayPanel(allOptions, frame);
        final JScrollPane scrollPane = new JScrollPane(cd);
        
        frame.getContentPane().setLayout(new BoxLayout(frame.getContentPane(), javax.swing.BoxLayout.X_AXIS));
        frame.getContentPane().add(scrollPane);
        frame.pack();
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

        return cd;
    }
}
